#include <wx/wx.h>
#include "canvas.h"
#include "dialogs.h"
#include "maptool.h"
#include <soil.h>
#include "glfont.h"

#include "mainFrame.h"

Vector3d contextPos;

BEGIN_EVENT_TABLE(Canvas3D, wxGLCanvas)
    EVT_SIZE(Canvas3D::OnSize)
    EVT_PAINT(Canvas3D::OnPaint)
    EVT_ENTER_WINDOW(Canvas3D::OnEnterWindow)
	EVT_LEAVE_WINDOW(Canvas3D::OnLeaveWindow)

	EVT_MOTION(Canvas3D::mouseMove)
	EVT_MOUSEWHEEL(Canvas3D::mouseWheel)
	EVT_LEFT_DOWN(Canvas3D::mouseClick)
	EVT_LEFT_UP(Canvas3D::mouseRelease)
	EVT_RIGHT_DOWN(Canvas3D::mouseRClick)

	EVT_KEY_DOWN(Canvas3D::keyPress)
	EVT_KEY_UP(Canvas3D::keyRelease)

	EVT_MENU(ID_CTX_CREATE_LOC, Canvas3D::createLoc)
	EVT_MENU(ID_CTX_CREATE_CP, Canvas3D::createCP)
	EVT_MENU(ID_CTX_CREATE_CON, Canvas3D::createCon)
	EVT_MENU(ID_CTX_DEL_LOC, Canvas3D::delLoc)
	EVT_MENU(ID_CTX_DEL_CP, Canvas3D::delCP)
	EVT_MENU(ID_CTX_DEL_CON, Canvas3D::delCon)
	EVT_MENU(ID_CTX_PLACE_LOC, Canvas3D::placeLoc)
END_EVENT_TABLE()

Canvas3D::Canvas3D(wxWindow *parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style, const wxString& name)
    : wxGLCanvas(parent, id, nullptr, pos, size, style|wxFULL_REPAINT_ON_RESIZE , name )
	 , context(new wxGLContext(this)) {}

Canvas3D::~Canvas3D() {
	glFont::Cleanup();
	delete context;
}

void Canvas3D::OnEnterWindow(wxMouseEvent&) {SetFocus();}

void Canvas3D::OnLeaveWindow(wxMouseEvent& event) {GetParent()->SetFocus();}

void Canvas3D::OnPaint(wxPaintEvent&) {
	wxPaintDC dc(this);
	SetCurrent(*context);
	Render();
}

void doZoom(bool in, float xPos, float yPos) {
	Vector3d oldPos=win2Map(xPos,yPos);
	zoom *= in ? 1.2f : 1.0f/1.2f;
	limitZoom();
	scrollOffset -= win2Map(xPos,yPos) - oldPos;
}

void Canvas3D::keyRelease(wxKeyEvent& evt) {
	if (evt.GetKeyCode() == 'M') measurePoints.clear();
	Render();
}

void Canvas3D::keyPress(wxKeyEvent& evt) {
	Vector3d oldMapPos = (selectedMap) ? selectedMap->position : Vector3d(0,0,0);
	Vector3d oldMapSize = (selectedMap) ? selectedMap->size : Vector3d(0,0,0);

	if (evt.ShiftDown() && evt.ControlDown()) {
		if (selectedMap  && frame->modePanel->GetSelection()==2) {
			if (evt.GetKeyCode() == WXK_UP) selectedMap->size.y+=1.0/zoom;
			if (evt.GetKeyCode() == WXK_DOWN) selectedMap->size.y-=1.0/zoom;
			if (evt.GetKeyCode() == WXK_RIGHT) selectedMap->size.x+=1.0/zoom;
			if (evt.GetKeyCode() == WXK_LEFT) selectedMap->size.x-=1.0/zoom;
		}
	}
	else if (evt.ControlDown()) {
		if (selectedMap && frame->modePanel->GetSelection()==2) {
			if (evt.GetKeyCode() == WXK_UP) selectedMap->position.y+=1.0/zoom;
			if (evt.GetKeyCode() == WXK_DOWN) selectedMap->position.y-=1.0/zoom;
			if (evt.GetKeyCode() == WXK_RIGHT) selectedMap->position.x+=1.0/zoom;
			if (evt.GetKeyCode() == WXK_LEFT) selectedMap->position.x-=1.0/zoom;
		}
	}
	else if (evt.ShiftDown()) {
		if (selectedMap && frame->modePanel->GetSelection()==2) {
			double ratio = selectedMap->size.x/selectedMap->size.y;
			if (evt.GetKeyCode() == WXK_UP)	{
				selectedMap->size.y += 1.0/zoom;
				selectedMap->size.x = selectedMap->size.y*ratio;
			}			
			if (evt.GetKeyCode() == WXK_DOWN) {
				selectedMap->size.y -= 1.0/zoom;
				selectedMap->size.x = selectedMap->size.y*ratio;
			}
			if (evt.GetKeyCode() == WXK_LEFT) {
				selectedMap->size.x -= 1.0/zoom;
				selectedMap->size.y = selectedMap->size.x/ratio;
			}
			if (evt.GetKeyCode() == WXK_RIGHT) {
				selectedMap->size.x += 1.0/zoom;
				selectedMap->size.y = selectedMap->size.x/ratio;
			}
		}
	}
	else {
		if (evt.GetKeyCode() == WXK_LEFT)	scrollOffset.x -= 25/zoom;
		if (evt.GetKeyCode() == WXK_RIGHT)	scrollOffset.x +=  25/zoom;
		if (evt.GetKeyCode() == WXK_UP)		scrollOffset.y +=  25/zoom;
		if (evt.GetKeyCode() == WXK_DOWN)	scrollOffset.y -=  25/zoom;
	}

	if (lockPointsToMap && selectedMap && (selectedMap->position!=oldMapPos || selectedMap->size!=oldMapSize)) {
		Vector3d delta=selectedMap->position - oldMapPos;
		Vector3d dsize=selectedMap->size - oldMapSize;
		list<Vector3d*>::iterator it=selectedPoints.begin();
		for (; it!=selectedPoints.end(); it++) {
			(**it)+=delta;
			(*it)->x += dsize.x * ((*it)->x - selectedMap->position.x) / oldMapSize.x;
			(*it)->y += dsize.y * ((*it)->y - selectedMap->position.y) / oldMapSize.y;
		}
	}

	if (evt.GetKeyCode() == 'L') lockPointsToMap=!lockPointsToMap;
	if (evt.GetKeyCode() == WXK_TAB && selectedMap) selectedMap->transparent=!selectedMap->transparent;
	if (evt.GetKeyCode() == WXK_PAGEUP)	  doZoom(1, vportW>>1, vportH>>1);
	if (evt.GetKeyCode() == WXK_PAGEDOWN) doZoom(0, vportW>>1, vportH>>1);

	if (evt.GetKeyCode() == WXK_DELETE && frame->modePanel->GetSelection()==2) {
		if (selectedCon && selectedWP>0 && selectedWP>0 && selectedWP<(int)selectedCon->via.size()-1) {
				selectedCon->via.erase( selectedCon->via.begin()+selectedWP );
				if (selectedWP >= (int)selectedCon->via.size()-1) selectedWP--;
				selectedCon->distance = selectedCon->length();
		}
		else if (selectedCon) {
			Connection::remove(selectedCon);
			selectedCon=0;
		}
		else if (selectedLoc) {
			Location::remove(selectedLoc);
			selectedLoc=0;
		}
		frame->selectionChanged();
	}
	Render();
}

void Canvas3D::Render() {
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	drawMap( frame->modePanel->GetSelection()==2 );
	SwapBuffers();
}

void Canvas3D::mouseWheel(wxMouseEvent& event) {
	doZoom(event.GetWheelRotation()>0, event.GetX(), event.GetY());
	Render();
}

void Canvas3D::mouseMove(wxMouseEvent& event) {
	int x=event.GetX(), y=event.GetY();
	static int oldX = x;
	static int oldY = y;

	mapPos = win2Map(x,y);

	if (event.Dragging() && event.MiddleIsDown()) 
		scrollOffset -= win2Map(x,y) - win2Map(oldX,oldY);

	if (event.Dragging() && event.LeftIsDown() && frame->modePanel->GetSelection()==2) {
		if (selectedCon && selectedWP>0) {
			selectedCon->via[selectedWP] = win2Map(x,y);
			selectedCon->distance = selectedCon->length();
			frame->updateConData();
		}
		else if (dragLocation && selectedLoc) {
			selectedLoc->placeAt(mapPos);
			frame->updateLocData();
		}
	}

	oldX = x;
	oldY = y;
	Render();
}

void Canvas3D::mouseRelease(wxMouseEvent& evt) {
	Vector3d mp = win2Map(evt.GetX(), evt.GetY());

	if (evt.ShiftDown() && frame->modePanel->GetSelection()==2) {
		Location* loc = Location::pick(mp);
		if (startConnect && loc && startConnect!=loc)
			selectedCon = Connection::add(startConnect, loc);
	}
	startConnect=0;
	dragLocation=false;
	if (!wxGetKeyState(wxKeyCode('M'))) measurePoints.clear();
	frame->selectionChanged();
}

void Canvas3D::mouseClick(wxMouseEvent& evt) {
	Vector3d mp = win2Map(evt.GetX(), evt.GetY());

	if (wxGetKeyState(wxKeyCode('M'))) {
		measurePoints.push_back(mp);
	}

	if (editMode) {
		if (evt.AltDown()) {
			selectedPoints.clear();
			selectedMap = Map::pick(mp);
			if (selectedMap)
				Location::getPointsInRect(selectedMap->position, selectedMap->position+selectedMap->size, selectedPoints);
		}
		else if (evt.ControlDown()) {
			if (selectedLoc) {
				selectedLoc->placeAt(mp);
				frame->updateLocData();
			}
		}
		else if (evt.ShiftDown()) {
			if (frame->modePanel->GetSelection()==2) startConnect = Location::pick(mp);
			if (startConnect) selectedWP=-1;
		}
		else {
			if (startConnect) {
				Location* loc = Location::pick(mp);
				if (startConnect && loc && startConnect!=loc) 
					selectedCon = Connection::add(startConnect, loc);
				startConnect=0;
			}
			else {
				selectedWP = selectedCon ? selectedCon->pickControlPoint(mp) : -1;
				if (!editMode) selectedWP=-1;
				dragLocation=false;

				if (selectedWP<=0) {
					selectedLoc = Location::pick(mp);
					if (selectedLoc) {
						dragLocation=true;
						frame->choiceFrom->SetStringSelection(selectedLoc->name);
						frame->editSelect->SetStringSelection(selectedLoc->name);
					}
					else if (frame->modePanel->GetSelection()==2) selectedCon = Connection::pick(mp);
			
					frame->updateLocData();
					frame->selectionChanged();
				}
			}
		}
	}
	else {
		Location* clicked = Location::pick(mp);
		if (evt.ShiftDown() && clicked) via.push_front(clicked);
		else {
			selectedLoc = clicked;
			if (selectedLoc) {
				frame->choiceFrom->SetStringSelection(selectedLoc->name);
				frame->editSelect->SetStringSelection(selectedLoc->name);
			}
			frame->updateLocData();
		}
		frame->selectionChanged();
	}
	Render();
}

int contextCP=0;
Connection* contextCon=0;
Location* contextLoc=0;

void Canvas3D::mouseRClick(wxMouseEvent& evt) {
	Vector3d mp = win2Map(evt.GetX(), evt.GetY());

	if (evt.ControlDown() && frame->modePanel->GetSelection()==2) {
		wxTextEntryDialog dia(this, LANG["enter_name_box"], LANG["enter_name"]);
		if (dia.ShowModal() == wxID_OK) Location::create(dia.GetValue().ToStdString(), mp);
		frame->selectionChanged();
	}
	else if (evt.ShiftDown() && frame->modePanel->GetSelection()==2) {
		if (selectedCon) selectedCon->insertCP(mp);
		Render();
	}
	else if (frame->modePanel->GetSelection()!=2) {
		selectedGoal = Location::pick(mp);
		if (selectedGoal) frame->choiceTo->SetStringSelection(selectedGoal->name);
		via.clear();
		frame->selectionChanged();
	}
	else {
		contextPos = mp;
		wxMenu nodeContext;
		contextCP = selectedCon ? selectedCon->pickControlPoint(mp) : -1;
		contextLoc = contextCP<1 ? Location::pick(mp) : 0;
		contextCon = (contextCP<1 && !contextLoc) ? Connection::pick(mp) : 0;
		bool sel = contextCP>0 || contextLoc || contextCon;

		if (!contextLoc) {
			nodeContext.Append(new wxMenuItem(&nodeContext, ID_CTX_CREATE_LOC, LANG["c_create_loc"]));
			if (contextCP<1 && selectedCon) 
				nodeContext.Append(new wxMenuItem(&nodeContext, ID_CTX_CREATE_CP, LANG["c_insert_cp"]));
		}
		if (!sel) {
			if (selectedLoc) nodeContext.Append(new wxMenuItem(&nodeContext, ID_CTX_PLACE_LOC, LANG["c_place_loc"]));
		}
		else {
			if (sel && !contextLoc) nodeContext.AppendSeparator();
			if (contextLoc) {
				nodeContext.Append(new wxMenuItem(&nodeContext, ID_CTX_CREATE_CON, LANG["c_connect_loc"]));
				nodeContext.AppendSeparator();
			}
			if (contextCP>0) nodeContext.Append(new wxMenuItem(&nodeContext, ID_CTX_DEL_CP, LANG["c_delete_cp"]));
			if (contextLoc) nodeContext.Append(new wxMenuItem(&nodeContext, ID_CTX_DEL_LOC, LANG["c_delete_loc"]));
			if (contextCon) nodeContext.Append(new wxMenuItem(&nodeContext, ID_CTX_DEL_CON, LANG["c_delete_con"]));
		}
		PopupMenu(&nodeContext);
		SetFocus();
	}
}

void Canvas3D::createLoc(wxCommandEvent& evt) {
	wxTextEntryDialog dia(this, LANG["enter_name_box"], LANG["enter_name"]);
	if (dia.ShowModal() == wxID_OK) Location::create(dia.GetValue().ToStdString(), contextPos);
	frame->selectionChanged();
}

void Canvas3D::createCP(wxCommandEvent& evt){
	if (selectedCon) selectedCon->insertCP(contextPos);
	Render();
}

void Canvas3D::createCon(wxCommandEvent& evt) {
	if (contextLoc) startConnect=contextLoc;
}

void Canvas3D::delLoc(wxCommandEvent& evt) {
	if (contextLoc) {
		if (selectedLoc==contextLoc) selectedLoc=0;
		Location::remove(contextLoc);
		frame->selectionChanged();
	}
}

void Canvas3D::delCP(wxCommandEvent& evt) {
	if (selectedCon) selectedCon->deleteCP(contextCP);
	frame->selectionChanged();
}

void Canvas3D::delCon(wxCommandEvent& evt) {
	if (contextCon) {
		if (selectedCon==contextCon) selectedCon=0;
		Connection::remove(contextCon);
		frame->selectionChanged();
	}
}

void Canvas3D::placeLoc(wxCommandEvent& evt) {
	if (selectedLoc) {
		selectedLoc->placeAt(contextPos);
		frame->updateLocData();
	}
}

void Canvas3D::OnSize(wxSizeEvent& event) {
	SetCurrent(*context);
	GetClientSize(&vportW, &vportH);
	glViewport(0,0, vportW,vportH);
	limitZoom();
	Refresh();
}

void Canvas3D::InitGL() {
	SetCurrent(*context);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_BLEND);
	iconTexture = SOIL_load_OGL_texture("data/textures/icons.dds", 0, 0, SOIL_FLAG_DDS_LOAD_DIRECT);
	glFont::Init("data/verdanasmall.glf");

	for (unsigned i=0; i<Map::maps.size(); i++)
		Map::maps[i].texture = SOIL_load_OGL_texture(Map::maps[i].textureFile.c_str(), 0, 0, SOIL_FLAG_DDS_LOAD_DIRECT);
	
	scrollOffset = totalPos+.5*(totalSize) - Vector3d(vportW>>1,vportH>>1,0)*zoom;	
}
