#include "curves.h"
#include <wx/glcanvas.h>

float step=.1f;

double distPointLineSegSq(const Vector3d& p, const Vector3d& a, const Vector3d& b) {
	Vector3d ap = p-a;
	Vector3d ab = b-a;
	float dist=0;

	if (ab.lenSq()==0) dist = a.distanceSq(p);
	else {
		double t = (ap*ab) / ab.lenSq();
		if (t<0) dist = a.distanceSq(p);
		else if (t>1) dist = b.distanceSq(p);
		else dist = p.distanceSq( a + t*ab );
	}
	return dist;
}

Vector3d getClosestPtLineSeg(const Vector3d& p, const Vector3d& a, const Vector3d& b) {
	Vector3d ap = p-a;
	Vector3d ab = b-a;
	Vector3d res;

	if (ab.lenSq()==0) res = a;
	else {
		double t = (ap*ab) / ab.lenSq();
		if (t<0) res = a;
		else if (t>1) res = b;
		else res = a + t*ab;
	}
	return res;
}

Vector3d CRGetClosestPt(const Vector3d& p, const Vector3d& p1, const Vector3d& p2, const Vector3d& p3, const Vector3d& p4) {
	Vector3d lastPos = catmullRom(0, p1, p2, p3, p4);
	Vector3d v, res = p1;
	float d, minDist = (p-lastPos).lenSq();

	for (float t=step; t<1.0; t+=step) {
		Vector3d pos = catmullRom(t, p1, p2, p3, p4);
		v = getClosestPtLineSeg(p, pos, lastPos);
		d = v.distanceSq(p);
		if (d < minDist) {
			minDist=d;
			res = v;
		}
		lastPos = pos;
	}
	v = getClosestPtLineSeg(p, catmullRom(1.0f, p1, p2, p3, p4), lastPos);
	d = v.distanceSq(p);
	
	return (d < minDist) ? v : res;
}

float CRSplinePointDistSq(const Vector3d& p, const Vector3d& p1, const Vector3d& p2, const Vector3d& p3, const Vector3d& p4) {
	Vector3d lastPos = catmullRom(0, p1, p2, p3, p4);
	float d, dist = (p-lastPos).lenSq();

	for (float t=step; t<1.0; t+=step) {
		Vector3d pos = catmullRom(t, p1, p2, p3, p4);
		d = distPointLineSegSq(p, pos, lastPos);
		if (d<dist) dist=d;
		lastPos = pos;
	}
	d = distPointLineSegSq(p, catmullRom(1.0f, p1, p2, p3, p4), lastPos);

	return (d<dist) ? d : dist;
}

Vector3d catmullRom(float t, const Vector3d& p1, const Vector3d& p2, const Vector3d& p3, const Vector3d& p4) {
	float tt=t*t;
	float ttt=t*tt;
	return .5*(p1*(-t+2.0*tt-ttt) + p2*(2.0-5.0*tt+3.0f*ttt) + p3*(t+4.0*tt-3.0*ttt) + p4*(-tt+ttt));
}

float CRSplineLength(const Vector3d& p1, const Vector3d& p2, const Vector3d& p3, const Vector3d& p4) {
	float len=0;
	Vector3d lastPos = catmullRom(0, p1, p2, p3, p4);

	for (float t=step; t<1.0; t+=step) {
		Vector3d pos = catmullRom(t, p1, p2, p3, p4);
		len += lastPos.distance( pos );
		lastPos = pos;
	}
	len += lastPos.distance( catmullRom(1.0f, p1, p2, p3, p4) );
	return len;
}

void renderCRSpline(const Vector3d& p1, const Vector3d& p2, const Vector3d& p3, const Vector3d& p4) {
	for (float t=.0; t<1.0; t+=step) 
		glVertex3dv( catmullRom(t, p1, p2, p3, p4) );
	glVertex3dv( catmullRom(1.0f, p1, p2, p3, p4) );
}
