#ifndef LOCATIONS_INCLUDED
#define LOCATIONS_INCLUDED

#include <string>
#include <list>
#include <map>

#include "vector.h"

using std::string;

class Connection;
class Transportation;

class Location {
public:
	static std::string locationFile;
	static std::map<string, Location*> locations;
	static std::list<Location*> waypoints;

	static Location* pick(const Vector3d& pos);
	static Location* get(const string& s);
	static Location* get(const int id);	
	static Location* add(const string& s, double x=0, double y=0);
	static Location* addWP(double x=0, double y=0);
	static void getPointsInRect(const Vector3d& min, const Vector3d& max, std::list<Vector3d*>& l);
	
	static void create(const string& name, const Vector3d& mp);
	static void remove(Location*);
	static void clear();
	static int numWaypoints;

	static void load(const string& file);
	static void save();

	Location(const string& s, double ix=0, double iy=0) : name(s), pos(ix,iy,0), wpID(-1) {}

	string name;
	Vector3d pos;
	int wpID;
	
	std::multimap<Location*, Connection*> connections;

	Location* prev;
	Connection* prCon;
	const Transportation* prTrans;
	double cfs, ctg;

	void placeAt(const Vector3d& p);
	bool rename(const string& nname);
	bool isInRect(const Vector3d& min, const Vector3d& max);
};

typedef std::pair<Location*, Connection*> ConEntry;

#endif
