#include "mainFrame.h"
#include "canvas.h"
#include "maptool.h"
#include <soil.h>
#include <wx/aboutdlg.h>

#include "logo.xpm"

using std::map;
using std::multimap;

MyFrame* frame=0;

BEGIN_EVENT_TABLE(MyFrame, wxFrame)
	EVT_CHOICE(ID_EDIT_SELECT, MyFrame::editLocSelected)
	EVT_CHOICE(ID_DEST_SELECT, MyFrame::planSelected)
	EVT_CHOICE(ID_FROM_SELECT, MyFrame::planSelected)
	EVT_CHOICE(ID_TYPE_BOX, MyFrame::typeChange)

	EVT_CHECKBOX(ID_FORCED_MARCH, MyFrame::forcedMarchChange)
	EVT_CHECKBOX(ID_FASTEST_ROUTE, MyFrame::fastestRouteChange)
	EVT_CHECKBOX(ID_TERRAIN_CB, MyFrame::terrainTypeCheck)
	EVT_CHECKBOX(ID_TRANS_CHECK, MyFrame::transportCheck)

	EVT_NOTEBOOK_PAGE_CHANGED(ID_MODE_BOOK, MyFrame::changeMode)

	EVT_MENU(ID_LOAD_DATA, MyFrame::onLoadData)
	EVT_MENU(ID_SAVE_DATA, MyFrame::onSaveData)
	EVT_MENU(ID_QUIT, MyFrame::quit)
	EVT_MENU(ID_ABOUT, MyFrame::about)

	EVT_BUTTON(ID_DEL_CONNECT, MyFrame::delConnection)
	EVT_BUTTON(ID_DEL_LOCATION, MyFrame::delLocation)
	EVT_BUTTON(ID_DELETE_CP, MyFrame::delControlPoint)
	EVT_BUTTON(ID_CON_REVERSE, MyFrame::conReverse)

	EVT_TEXT_ENTER(ID_NAME_EDIT, MyFrame::changeLocName)
	EVT_TEXT_ENTER(ID_MODIFIER_TXT, MyFrame::changeConMod)
	EVT_TEXT_ENTER(ID_DISTANCE_TXT, MyFrame::changeConDist)
END_EVENT_TABLE()


MyFrame::MyFrame( wxWindow* parent, int id, wxString title, wxPoint pos, wxSize size, int style ) 
: wxFrame(parent, id, title, pos, size, style), canvas(0) {

	SetIcon( logo_xpm );

	wxMenuBar* m_menubar1 = new wxMenuBar(0);

	wxMenu* m_menu1 = new wxMenu();
	m_menu1->Append( new wxMenuItem( m_menu1, ID_LOAD_DATA, LANG["load"]+"\tCTRL+L") );
	m_menu1->Append( new wxMenuItem( m_menu1, ID_SAVE_DATA, LANG["save"]+"\tCTRL+S") );
	
	m_menu1->AppendSeparator();
	m_menu1->Append( new wxMenuItem( m_menu1, ID_QUIT, LANG["exit"]+"\tALT+F4") );
	m_menubar1->Append(m_menu1, LANG["file"]);

	wxMenu* m_menu2 = new wxMenu();
	m_menu2->Append( new wxMenuItem( m_menu2, ID_ABOUT, LANG["about"]+"\tF1") );
	m_menubar1->Append(m_menu2, LANG["help"]);

	SetMenuBar(m_menubar1);


	modePanel = new wxNotebook(this, ID_MODE_BOOK, wxDefaultPosition, wxDefaultSize, wxSIMPLE_BORDER);

	wxBoxSizer* bSizer1 = new wxBoxSizer(wxHORIZONTAL);	
	wxBoxSizer* bSizer2 = new wxBoxSizer(wxVERTICAL);
	wxBoxSizer* pbs = new wxBoxSizer(wxVERTICAL);

	wxFlexGridSizer* fgSizer1 = new wxFlexGridSizer(2, 2, 0, 0);
	fgSizer1->AddGrowableCol(1);
	fgSizer1->SetFlexibleDirection(wxHORIZONTAL);

	wxPanel* tmpPanel = new wxPanel(modePanel, ID_DEFAULT);
	modePanel->AddPage(tmpPanel, LANG["plan"], false);

	fgSizer1->Add(new wxStaticText(tmpPanel, -1, LANG["from"]), 0, wxALL|wxALIGN_CENTER_VERTICAL, 5);
	
	choiceFrom = new wxChoice(tmpPanel, ID_FROM_SELECT);
	fgSizer1->Add(choiceFrom, 0, wxALL|wxEXPAND, 5);
	
	fgSizer1->Add(new wxStaticText(tmpPanel, -1, LANG["to"]), 0, wxALL|wxALIGN_CENTER_VERTICAL, 5);
	
	choiceTo = new wxChoice(tmpPanel, ID_DEST_SELECT);
	fgSizer1->Add(choiceTo, 0, wxALL|wxEXPAND, 5);

	pbs->Add(fgSizer1, 0, wxALL|wxEXPAND, 5);

	wxBoxSizer* routeBox = new wxBoxSizer(wxHORIZONTAL);

	wxCheckBox* tmpCheck = new wxCheckBox(tmpPanel, ID_FORCED_MARCH, LANG["forced"]);
	tmpCheck->SetValue(doForcedMarch);
	routeBox->Add(tmpCheck, 1, wxALL, 5);

	tmpCheck = new wxCheckBox(tmpPanel, ID_FASTEST_ROUTE, LANG["fastest"]);
	tmpCheck->SetValue(fastestRoute);
	routeBox->Add(tmpCheck, 0, wxALL, 5);

	pbs->Add(routeBox, 0, wxALL|wxEXPAND, 0);


	wxStaticBoxSizer* vehSelect = new wxStaticBoxSizer( new wxStaticBox(tmpPanel, -1, LANG["allowed_transport"]), wxVERTICAL);
	vehWindow = new wxScrolledWindow( tmpPanel, ID_DEFAULT, wxDefaultPosition, wxSize(-1, 150), wxVSCROLL );
	vehWindow->SetScrollRate(0, 15);
	vehSelect->Add(vehWindow, 0, wxEXPAND);

	vhwBox = new wxBoxSizer(wxVERTICAL);
	vehWindow->SetSizer(vhwBox);

	pbs->Add(vehSelect, 0, wxALL|wxEXPAND, 5);
	
	routeLog = new wxTextCtrl(tmpPanel, -1, "", wxDefaultPosition, wxDefaultSize, wxTE_READONLY|wxTE_MULTILINE);
	pbs->Add(routeLog, 1, wxALL|wxEXPAND, 5);

	tmpPanel->SetSizer(pbs);
	tmpPanel->Layout();
	pbs->Fit(tmpPanel);

	tmpPanel = new wxPanel(modePanel, ID_DEFAULT);
	modePanel->AddPage(tmpPanel, LANG["travel"], false);




	tmpPanel = new wxPanel(modePanel, ID_DEFAULT);
	modePanel->AddPage(tmpPanel, LANG["edit"], false);

	wxBoxSizer* bSizer6 = new wxBoxSizer(wxVERTICAL);
	
	wxBoxSizer* bSel = new wxBoxSizer(wxHORIZONTAL);

	editSelect = new wxChoice(tmpPanel, ID_EDIT_SELECT); 
	bSel->Add(editSelect, 1, wxALL, 5);

	wxButton* tmpBtn = new wxButton(tmpPanel, ID_DEL_LOCATION, LANG["del"], wxDefaultPosition, wxSize(40, -1));
	bSel->Add(tmpBtn, 0, wxALL, 5);

	bSizer6->Add(bSel, 0, wxALL|wxEXPAND, 0);

	wxBoxSizer* bSizerX1 = new wxBoxSizer(wxHORIZONTAL);
	bSizerX1->Add(new wxStaticText(tmpPanel, -1, LANG["name"]) ,0, wxALL|wxALIGN_CENTER_VERTICAL, 5);
	editNameCtrl = new wxTextCtrl(tmpPanel, ID_NAME_EDIT, "", wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER);
	bSizerX1->Add(editNameCtrl, 1, wxALL, 5);	
	bSizer6->Add(bSizerX1, 0, wxEXPAND, 5);

	bSizer6->Add(new wxStaticLine(tmpPanel), 0, wxEXPAND|wxALL, 5);
	
	

	wxBoxSizer* bSizercon = new wxBoxSizer(wxHORIZONTAL);
	conInfo = new wxStaticText(tmpPanel, -1, "");
	bSizercon->Add(conInfo, 1, wxALL|wxALIGN_CENTER_VERTICAL, 5);

	tmpBtn = new wxButton(tmpPanel, ID_DEL_CONNECT, LANG["del"], wxDefaultPosition, wxSize(40, -1));
	bSizercon->Add(tmpBtn, 0, wxALL, 5);

	bSizer6->Add(bSizercon, 0, wxALL|wxEXPAND, 0);


	wxBoxSizer* bSizerst = new wxBoxSizer(wxHORIZONTAL);

	wxBoxSizer* fgSizer2 = new wxBoxSizer(wxVERTICAL);
	fgSizer2->Add(new wxStaticText(tmpPanel, -1, LANG["length"]), 0, wxALL, 5);
	conDistTxt = new wxTextCtrl(tmpPanel, ID_DISTANCE_TXT, "", wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER);
	fgSizer2->Add(conDistTxt, 0, wxEXPAND|wxLEFT|wxBOTTOM, 5);

	
	wxBoxSizer* bSizerht = new wxBoxSizer(wxHORIZONTAL);

	wxBoxSizer* bSizervt = new wxBoxSizer(wxVERTICAL);
	bSizervt->Add(new wxStaticText(tmpPanel, -1, LANG["modifier"]), 0, wxALL, 5);
	conModTxt = new wxTextCtrl(tmpPanel, ID_MODIFIER_TXT, "", wxDefaultPosition, wxSize(40,-1), wxTE_PROCESS_ENTER);
	bSizervt->Add(conModTxt, 0, wxEXPAND|wxLEFT|wxBOTTOM, 5);
	
	bSizerht->Add(bSizervt, 0);

	wxBoxSizer* bSizervt2 = new wxBoxSizer(wxVERTICAL);

	bSizervt2->Add( new wxStaticText(tmpPanel, -1, LANG["type"]), 0, wxALL, 5);

	seaBox = new wxChoice(tmpPanel, ID_TYPE_BOX);
	seaBox->Append(LANG["land"]);
	seaBox->Append(LANG["sea"]);
	seaBox->Append(LANG["river"]);
	bSizervt2->Add(seaBox, 0, wxLEFT|wxBOTTOM, 5);

	bSizerht->Add(bSizervt2, 0);

	fgSizer2->Add(bSizerht, 0, wxALL, 0);
	bSizerst->Add(fgSizer2, 0);


	wxBoxSizer* bSize50 = new wxBoxSizer(wxVERTICAL);
	bSize50->Add( new wxStaticText(tmpPanel, -1, ""), 0, wxTOP, 5);
	tmpBtn = new wxButton(tmpPanel, ID_CON_REVERSE, LANG["reverse"]);
	bSize50->Add(tmpBtn, 0, wxALL, 5);

	bSize50->Add( new wxStaticText(tmpPanel, -1, ""), 0, wxTOP, 5);
	tmpBtn = new wxButton(tmpPanel, ID_DELETE_CP, LANG["del_cp"]);
	bSize50->Add(tmpBtn, 0, wxALL, 5);
	
	bSizerst->Add(bSize50, 0, wxLEFT, 5);	
	bSizer6->Add(bSizerst, 0);




	wxStaticBoxSizer* sbSizerip = new wxStaticBoxSizer(new wxStaticBox(tmpPanel, -1, LANG["impassable"] ), wxVERTICAL);
	wxFlexGridSizer* fgSizerip = new wxFlexGridSizer(2, 3, 0, 0);
	fgSizerip->SetFlexibleDirection(wxBOTH);
	fgSizerip->AddGrowableCol(2);

	fgSizerip->Add(new wxStaticText(tmpPanel, -1, LANG["imp_from"]) ,0, wxALL|wxALIGN_CENTER_VERTICAL, 5);
	
	wxChoice* tmpChoice = new wxChoice(tmpPanel, -1);
	tmpChoice->AppendString("15");
	fgSizerip->Add(tmpChoice, 0, wxALL, 5);
	
	tmpChoice = new wxChoice(tmpPanel, -1);
	tmpChoice->AppendString("Rondra");
	fgSizerip->Add(tmpChoice, 0, wxALL|wxEXPAND, 5);

	fgSizerip->Add(new wxStaticText(tmpPanel, -1, LANG["imp_until"]) ,0, wxALL|wxALIGN_CENTER_VERTICAL, 5);
	
	tmpChoice = new wxChoice(tmpPanel, -1); 
	tmpChoice->AppendString("15");
	fgSizerip->Add(tmpChoice, 0, wxALL, 5);
	
	tmpChoice = new wxChoice(tmpPanel, -1); 
	tmpChoice->AppendString("Peraine");
	fgSizerip->Add(tmpChoice, 0, wxALL|wxEXPAND, 5);

	sbSizerip->Add(fgSizerip, 0, wxALL|wxEXPAND, 0);
	bSizer6->Add(sbSizerip, 0, wxALL|wxEXPAND, 5);



	wxStaticBoxSizer* sbSizer1 = new wxStaticBoxSizer( new wxStaticBox(tmpPanel, -1, LANG["terrain_type"]), wxVERTICAL);
	ttSizer = new wxBoxSizer(wxVERTICAL);
	
	sWindow = new wxScrolledWindow( tmpPanel, ID_DEFAULT, wxDefaultPosition, wxDefaultSize, wxVSCROLL );
	sWindow->SetScrollRate(0, 15);
	sWindow->SetSizer(ttSizer);

	sbSizer1->Add(sWindow, 1, wxEXPAND|wxALL, 0);
	bSizer6->Add(sbSizer1, 1, wxALL|wxEXPAND, 5);

	ttSizer->Fit(sWindow);
	sbSizer1->Fit(tmpPanel);
	
	
	tmpPanel->SetSizer(bSizer6);
	tmpPanel->Layout();
	bSizer6->Fit(tmpPanel);
	
	bSizer2->Add(modePanel, 1, wxEXPAND, 0);	
	bSizer1->Add(bSizer2, 0, wxEXPAND, 5);
	
	canvas = new Canvas3D(this, ID_DEFAULT, wxDefaultPosition, wxDefaultSize, wxSIMPLE_BORDER);
	bSizer1->Add(canvas, 1, wxEXPAND, 0);
	
	SetSizer(bSizer1);
	
	syncLists();

	Show(true);
	canvas->InitGL();
	Layout();
}

void MyFrame::quit(wxCommandEvent&) {Close(1);}

void MyFrame::about(wxCommandEvent&) {
	wxAboutDialogInfo info;
    info.SetName("DSA Route");
    info.SetVersion("0.9 Beta");
	info.SetDescription("\nLMB: select\n"
						"RMB: select goal\n"
						"Shift+LMB+Drag: connect locations\n"
						"Shift+RMB: insert control point\n"
						"Ctrl+LMB: move location\n"
						"Ctrl+RMB: create location");
    wxAboutBox(info);
}

void MyFrame::onLoadData(wxCommandEvent&) {
	wxFileDialog load(this, "Selected file", "data", "", "*.drf", wxFD_OPEN|wxFD_FILE_MUST_EXIST);
	if (wxID_OK == load.ShowModal()) {
		loadData(load.GetPath().ToStdString());

		for (unsigned i=0; i<Map::maps.size(); i++)
			Map::maps[i].texture = SOIL_load_OGL_texture(Map::maps[i].textureFile.c_str(), 0, 0, SOIL_FLAG_DDS_LOAD_DIRECT);

		scrollOffset = totalPos+.5*(totalSize) - Vector3d(vportW>>1,vportH>>1,0)*zoom;
		syncLists();
		selectionChanged();
	}
}

void MyFrame::onSaveData(wxCommandEvent&) {
	Location::save();
	Connection::save();
	Map::save();
}

void MyFrame::delConnection(wxCommandEvent&) {
	Connection::remove(selectedCon);
	selectedCon=0;
	selectionChanged();
}

void MyFrame::delLocation(wxCommandEvent&) {
	if (selectedLoc) {
		Location::remove(selectedLoc);
		selectedLoc=0;
		selectionChanged();
	}
}

void MyFrame::delControlPoint(wxCommandEvent&) {
	if (selectedCon) selectedCon->deleteCP(selectedWP);
	selectionChanged();
}

void MyFrame::updateRouteInfo() {
	routeLog->SetValue( routeToString() );
}

void MyFrame::fastestRouteChange(wxCommandEvent& evt) {
	fastestRoute=evt.IsChecked();
	planRoute(selectedLoc, selectedGoal, fastestRoute);
	canvas->Render();
	updateRouteInfo();
}

void MyFrame::forcedMarchChange(wxCommandEvent& evt) {
	doForcedMarch=evt.IsChecked();
	planRoute(selectedLoc, selectedGoal, fastestRoute);
	canvas->Render();
	updateRouteInfo();
}

void MyFrame::changeMode(wxNotebookEvent&) {
	editMode = (modePanel->GetSelection()==2);
	if (!editMode) selectedCon=0;
	selectionChanged();
}

void MyFrame::transportCheck(wxCommandEvent& evt) {
	unsigned i=0;
	for (i=0; i<Transportation::transports.size(); i++)
		Transportation::transports[i].allowed = landTransAllowed[i]->IsChecked();
	selectionChanged();
}

void MyFrame::planSelected(wxCommandEvent& evt) {
	string s=evt.GetString();
	Location* sel = Location::get(s);
	
	if (sel) {
		if (evt.GetId()==ID_FROM_SELECT) selectedLoc = sel;
		if (evt.GetId()==ID_DEST_SELECT) {
			via.clear();
			selectedGoal = sel;
		}
		planRoute(selectedLoc, selectedGoal, fastestRoute);
		
		updateRouteInfo();
		centerOnRoute();
		canvas->Render();
	}
}

void MyFrame::selectionChanged() {
	if (selectedLoc && selectedGoal && !editMode)
		planRoute(selectedLoc, selectedGoal, fastestRoute);
	else route.clear();

	if (canvas) {
		updateRouteInfo();
		updateLocData();
		canvas->Render();
	}
}

void MyFrame::editLocSelected(wxCommandEvent& evt) {
	string sel = evt.GetString();
	Location* l = Location::get(sel);
	if (l) {
		selectedLoc = l;
		selectionChanged();
	}
}

void MyFrame::refresh() {
	syncLists();
	editSelect->SetStringSelection(selectedLoc->name);
	updateLocData();
	canvas->Render();
}

void MyFrame::updateLocData() {
	if (selectedLoc) editNameCtrl->SetValue(selectedLoc->name);
	updateConData();
}

void MyFrame::updateConData() {
	if (selectedCon) {
		multimap<Location*, Connection*>::iterator it = selectedCon->from->connections.find(selectedCon->to);
		int count = (int)selectedCon->from->connections.count(selectedCon->to);
		int num=1;
		while (it != selectedCon->from->connections.end() && selectedCon != it->second) {
			num++; it++;
		}

		string from = selectedCon->from->name;
		if (from.empty()) from = wxString::Format("WP %d", selectedCon->from->wpID);
		
		string to = selectedCon->to->name;
		if (to.empty()) to = wxString::Format("WP %d", selectedCon->to->wpID);
		
		conInfo->SetLabel( wxString::Format("%s - %s (%d of %d)", 
			from.c_str(), to.c_str(), num, count) );

		conDistTxt->SetValue( wxString::Format("%f", selectedCon->distance) );
		conModTxt->SetValue( wxString::Format("%.2f", selectedCon->modifier) );
		seaBox->SetSelection( selectedCon->type );
		for (unsigned i=0; i<terTypes.size(); i++)
			terTypes[i]->SetValue( 0!=(selectedCon->terrainType&(1 << TerrainType::terrains[i].id)) );
	}
	else {
		conInfo->SetLabel("");
		conDistTxt->Clear();
		conModTxt->Clear();
		seaBox->SetSelection(0);
		for (unsigned i=0; i<terTypes.size(); i++) terTypes[i]->SetValue(0);
	}
}

void MyFrame::conReverse(wxCommandEvent&) {
	if (selectedCon) {
		selectedCon->reverse();
		selectionChanged();
	}
}

void MyFrame::syncLists() {
	choiceFrom->Clear();
	choiceTo->Clear();
	editSelect->Clear();

	map<string, Location*>::iterator it = Location::locations.begin();
	for (; it!=Location::locations.end(); it++) {
		choiceFrom->AppendString(it->first);
		choiceTo->AppendString(it->first);
		editSelect->AppendString(it->first);
	}

	if (selectedLoc) {
		choiceFrom->SetStringSelection(selectedLoc->name);
		editSelect->SetStringSelection(selectedLoc->name);
	}
	if (selectedGoal) {
		choiceTo->SetStringSelection(selectedGoal->name);
	}

	landTransAllowed.clear();
	vehWindow->DestroyChildren();
	
	vector<Transportation>::iterator tit=Transportation::transports.begin();
	for (; tit!=Transportation::transports.end(); tit++) {
		wxCheckBox* tmpCheck=0;

		if (tit->type==LAND)
			tmpCheck = new wxCheckBox(vehWindow, ID_TRANS_CHECK, 
			wxString::Format("%s (%.0f/%.2f)", tit->name.c_str(), tit->speed, tit->speed*tit->fastFac));
		
		if (tit->type==RIVER)
			tmpCheck = new wxCheckBox(vehWindow, ID_TRANS_CHECK, 
			wxString::Format("%s (%.0f/%.0f)", tit->name.c_str(), tit->speed, tit->revSpeed));
		
		if (tit->type==SEA)
			tmpCheck = new wxCheckBox(vehWindow, ID_TRANS_CHECK, 
			wxString::Format("%s (%.0f)", tit->name.c_str(), tit->speed));
		
		if (tmpCheck) {
			tmpCheck->SetValue(tit->allowed);
			vhwBox->Add(tmpCheck, 0, wxLEFT|wxTOP, 5);
			landTransAllowed.push_back(tmpCheck);
		}
	}
	vehWindow->Layout();

	terTypes.clear();
	sWindow->DestroyChildren();
	for (unsigned ti=0; ti<TerrainType::terrains.size(); ti++) {
		wxCheckBox* tmpCheck = new wxCheckBox(sWindow, ID_TERRAIN_CB, TerrainType::terrains[ti].name.c_str());
		ttSizer->Add(tmpCheck, 0, wxTOP|wxLEFT|wxRIGHT, 5);
		terTypes.push_back(tmpCheck);
	}
	sWindow->Layout();
}

void MyFrame::changeLocName(wxCommandEvent& evt) {
	if (selectedLoc && selectedLoc->name != evt.GetString()) {
		if (selectedLoc->rename(evt.GetString().ToStdString())) syncLists();
		else wxMessageBox("Location of that name already exists", "Error", wxOK);
	}
}

void MyFrame::typeChange(wxCommandEvent& evt) {
	if (selectedCon) {
		if (evt.GetSelection()==0) selectedCon->type=LAND;
		if (evt.GetSelection()==1) selectedCon->type=SEA;
		if (evt.GetSelection()==2) selectedCon->type=RIVER;
		updateConData();
	}
}

void MyFrame::terrainTypeCheck(wxCommandEvent& evt) {
	if (selectedCon) {
		wxCheckBox* cb = (wxCheckBox*)evt.GetEventObject();
		TerrainType* type = TerrainType::get(cb->GetLabel().ToStdString());
		if (cb->IsChecked()) selectedCon->terrainType |= 1<<type->id;
		else selectedCon->terrainType &= ~(1<<type->id);

		float totalMod=.0f;
		int numMods=0;
		for (unsigned tt=0; tt<TerrainType::terrains.size(); tt++) {
			if ((1<<TerrainType::terrains[tt].id) & selectedCon->terrainType) {
				totalMod+=TerrainType::terrains[tt].mod;
				numMods++;
			}
		}
		if (numMods>0) selectedCon->modifier = totalMod/numMods;
		else selectedCon->modifier = 1.0f;

		updateConData();
	}
}

void MyFrame::changeConMod(wxCommandEvent& evt) {
	if (selectedCon) {
		double x;
		if (evt.GetString().ToDouble(&x)) selectedCon->modifier = x;
		if (selectedCon->modifier == 0) selectedCon->modifier=.1f;
		updateConData();
	}
}

void MyFrame::changeConDist(wxCommandEvent& evt) {
	if (selectedCon) {
		double x;
		if (evt.GetString().ToDouble(&x)) selectedCon->distance = x;
		updateConData();
	}
}
