#ifndef __noname__
#define __noname__

#include <wx/wx.h>
#include <wx/choice.h>
#include <wx/panel.h>
#include <wx/notebook.h>
#include <wx/tglbtn.h>
#include <wx/statline.h>
#include <vector>

class Canvas3D;

const int ID_DEFAULT = wxID_ANY;
enum {ID_EDIT_SELECT = 1000, ID_SAVE_LOC_BTN, ID_NAME_EDIT, ID_MODE_BOOK,
		ID_DEST_SELECT, ID_FROM_SELECT, ID_MEANS_SELECT, ID_FORCED_MARCH, 
		ID_CONNECT_SELECT, ID_DISTANCE_TXT, ID_MODIFIER_TXT, ID_DEL_CONNECT,
		ID_INSERT_CP, ID_DELETE_CP, ID_DEL_LOCATION, ID_FASTEST_ROUTE, ID_TYPE_BOX,
		ID_LOAD_DATA, ID_SAVE_DATA, ID_QUIT, ID_ABOUT, ID_SHIP_SELECT, ID_BOAT_SELECT,
		ID_ALLOW_SHIP, ID_TERRAIN_CB, ID_CON_REVERSE, ID_TRANS_CHECK,
		
		ID_CTX_CREATE_LOC, ID_CTX_CREATE_CP, ID_CTX_CREATE_CON,
		ID_CTX_DEL_LOC, ID_CTX_DEL_CP, ID_CTX_DEL_CON,
		ID_CTX_PLACE_LOC
};


class MyFrame : public wxFrame {
	DECLARE_EVENT_TABLE()

	public:
		wxScrolledWindow *vehWindow, *sWindow;
		wxBoxSizer *vhwBox, *ttSizer;
		wxNotebook *modePanel;
		wxChoice *choiceFrom, *choiceTo, *editSelect;
		Canvas3D *canvas;
		wxTextCtrl *editNameCtrl, *routeLog;
		wxTextCtrl *conDistTxt, *conModTxt;
		wxChoice *seaBox;
		wxStaticText *conInfo;
		std::vector<wxCheckBox*> terTypes;
		std::vector<wxCheckBox*> landTransAllowed;
		std::vector<wxCheckBox*> seaTransAllowed;
		std::vector<wxCheckBox*> riverTransAllowed;

		MyFrame(wxWindow* parent, int id=-1, wxString title="DSA Route", wxPoint pos=wxDefaultPosition, wxSize size=wxSize(800, 600), int style = wxDEFAULT_FRAME_STYLE);
		
		void syncLists();
		void refresh();
		void updateLocData();
		void updateConData();
		void updateRouteInfo();
		void selectionChanged();
		
		void quit(wxCommandEvent&);
		void about(wxCommandEvent&);
		void onLoadData(wxCommandEvent&);
		void onSaveData(wxCommandEvent&);

		void changeConMod(wxCommandEvent&);
		void changeConDist(wxCommandEvent&);
		void terrainTypeCheck(wxCommandEvent&);
		void typeChange(wxCommandEvent&);
		void changeMode(wxNotebookEvent&);
		void transportCheck(wxCommandEvent& evt);
		
		void conReverse(wxCommandEvent&);
		void delConnection(wxCommandEvent&);
		void delControlPoint(wxCommandEvent&);
		void delLocation(wxCommandEvent&);
		void editLocSelected(wxCommandEvent& evt);
		void planSelected(wxCommandEvent& evt);

		void changeLocName(wxCommandEvent& evt);
		void fastestRouteChange(wxCommandEvent& evt);
		void forcedMarchChange(wxCommandEvent& evt);
};

extern MyFrame* frame;

#endif
