#include "maptool.h"

#include <wx/glcanvas.h>
#include <tinyxml.h>

vector<Map> Map::maps;

Map::~Map() {glDeleteTextures(1, &texture);}

Map* Map::pick(const Vector3d& mp) {
	for (int i=(int)maps.size()-1; i>=0; i--) {
		if (mp.x>maps[i].position.x && mp.x<maps[i].position.x+maps[i].size.x && 
					mp.y>maps[i].position.y && mp.y<maps[i].position.y+maps[i].size.y)
			return &maps[i];
	}
	return 0;
}

void Map::save() {
	for (unsigned i=0; i<maps.size(); i++) maps[i].saveMap();
}

void Map::saveMap() {
	TiXmlDocument xml(file.c_str());
	TiXmlElement* root = xml.InsertEndChild(TiXmlElement("mapDesc"))->ToElement();

	root->SetAttribute("texture", textureFile.c_str());
	root->SetDoubleAttribute("mapScale", scale);

	Vector3d center = -position * (1.0/scale);
	Vector3d sz = size * (1.0/scale);

	root->SetDoubleAttribute("mapWidth", sz.x);
	root->SetDoubleAttribute("mapHeight", sz.y);
	
	root->SetDoubleAttribute("coordCenterX", center.x);
	root->SetDoubleAttribute("coordCenterY", sz.y-center.y);
	
	xml.SaveFile();
}

Map* Map::load(const string& file) {
	TiXmlDocument xml(file.c_str());
	if (xml.LoadFile()) {
		TiXmlElement* root=xml.RootElement();
		if (root) {
			Map nmap;
			nmap.file = file;
			root->QueryValueAttribute("texture", &nmap.textureFile);

			root->QueryDoubleAttribute("mapWidth", &nmap.size.x);
			root->QueryDoubleAttribute("mapHeight", &nmap.size.y);
			root->QueryDoubleAttribute("mapScale", &nmap.scale);

			Vector3d center;
			root->QueryDoubleAttribute("coordCenterX", &center.x);
			root->QueryDoubleAttribute("coordCenterY", &center.y);				
			
			center.y = nmap.size.y - center.y;
			nmap.size *= nmap.scale;
			nmap.position = -nmap.scale*center;
			maps.push_back(nmap);
			return &maps.back();
		}
	}
	return 0;
}

void Map::draw() {
	if (editMode && selectedMap == this) glColor4f(1,.7f,.65f, transparent?.5f:1.0f);
	else glColor4f(1,1,1,transparent?.5f:1.0f);
	glBindTexture(GL_TEXTURE_2D, texture);
	glBegin(GL_QUADS);
		glTexCoord2f(1,0); glVertex2d((position+size).x, (position+size).y);
		glTexCoord2f(0,0); glVertex2d(position.x, position.y+size.y);
		glTexCoord2f(0,1); glVertex2d(position.x, position.y);
		glTexCoord2f(1,1); glVertex2d(position.x+size.x, position.y);
	glEnd();
	glColor4f(1,1,1,1);
}
