#include "maptool.h"
#include <wx/glcanvas.h>
#include "glfont.h"
#include <tinyxml.h>
#include "platform.h"

using std::map;

Location *selectedLoc=0, *selectedGoal=0;
Connection* selectedCon;
int selectedWP=-1;
Map* selectedMap=0;
list<Vector3d*> selectedPoints;

unsigned iconTexture=0;

Vector3d scrollOffset(.0,.0,.0);
Vector3d mapPos, totalPos, totalSize;

bool dragLocation=0;
Location* startConnect=0;

vector<Vector3d> measurePoints;

double zoom=1.0f;
int vportW=1, vportH=1;

bool lockPointsToMap=1;
bool editMode=0;

Vector3d win2Map(int x, int y) {return Vector3d(scrollOffset.x + x/zoom, scrollOffset.y + (vportH-y)/zoom, 0);}

void loadData(const string& file) {
	TiXmlDocument xml(file.c_str());
	if (xml.LoadFile()) {
		TiXmlElement* root=xml.RootElement();

		if (root) {
			const char* file;

			Map::maps.clear();
			selectedLoc=selectedGoal=startConnect=0;
			selectedCon=0;
			selectedMap=0;
			selectedPoints.clear();			
			route.clear();

			TiXmlElement* m=root->FirstChildElement("map");
			bool first=true;
			Vector3d min,max;
			while (m) {
				const char* file = m->GetText();
				if (file) {
					Map* map = Map::load(file);
					if (map) {
						Vector3d ll=map->position;
						Vector3d ur=map->position+map->size;
						if (first) {
							min=ll; max=ur;
							first=false;
						}
						else {
							if (ll.x<min.x) min.x=ll.x;
							if (ll.y<min.y) min.y=ll.y;
							if (ur.x>max.x) max.x=ur.x;
							if (ur.y>max.y) max.y=ur.y;
						}
					}
				}
				m = m->NextSiblingElement("map");
			}
			totalPos = min;
			totalSize = max-min;

			file = root->Attribute("terrainTypes");
			if (file) TerrainType::load(file);

			file = root->Attribute("locations");
			if (file) Location::load( file );

			file = root->Attribute("routes");
			if (file) Connection::load( file );

			file = root->Attribute("transportMeans");
			if (file) Transportation::load(file);
		}
	}
}

void limitZoom() {
	double hdiff = totalSize.y*zoom - vportH;
	double wdiff = totalSize.x*zoom - vportW;

	if (hdiff<0 && wdiff<0) {
		if (hdiff>wdiff) zoom = (double)vportH/totalSize.y;
		else zoom = (double)vportW/totalSize.x;
	}
}

void drawMap(bool showCons) {
	glPushMatrix();
	glTranslated(-scrollOffset.x, -scrollOffset.y, .0);

	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	
	glOrtho(0, vportW/zoom, 0, vportH/zoom, -1, 1);

	glEnable(GL_TEXTURE_2D);
	for (unsigned m=0; m<Map::maps.size(); m++) Map::maps[m].draw();

	glDisable(GL_TEXTURE_2D);
	glLineWidth(3.0f);
	glPointSize(7.0f);

	if (editMode) {
		list<Connection*>::iterator cit = Connection::connections.begin();
		for (; cit!=Connection::connections.end(); cit++) (*cit)->draw();
	
		if (selectedCon) {
			glColor3f(1,1,0);
			selectedCon->draw(true);

			glBegin(GL_POINTS);
			for (unsigned i=1; i<selectedCon->via.size()-1; i++) {
				if (selectedWP==i) glColor3f(1,0,1);
				else glColor3f(0,1,1);
				glVertex3dv( selectedCon->via[i] );
			}
			glEnd();
		}
	}

	if (!editMode) {
		glColor3f(1,0,0);
		list<SearchInfo>::iterator rit=route.begin();
		for (; rit!=route.end(); rit++)
			if (rit->prCon) rit->prCon->draw(true);
	}

	glBegin(GL_POINTS);
		glColor3f(.35f, .35f, .35f);
		list<Location*>::iterator wit=Location::waypoints.begin();
		for (; wit!=Location::waypoints.end(); wit++)
			glVertex3dv((*wit)->pos);
	
		glColor3f(1,1,1);
		map<string, Location*>::iterator it=Location::locations.begin();
		for (it=Location::locations.begin(); it!=Location::locations.end(); it++)
			glVertex3dv(it->second->pos);
	glEnd();

	if (startConnect) {
		glBegin(GL_LINES);
			glColor3f(.75f, .75f, .75f);
			glVertex3dv(startConnect->pos);
			glVertex3dv(mapPos);
		glEnd();
	}
	if (!measurePoints.empty()) {
		glColor3f(.95f, .95f, .95f);
		glBegin(GL_LINE_STRIP);
		for (unsigned mp=0; mp<measurePoints.size(); mp++) glVertex3dv(measurePoints[mp]);
		glVertex3dv(mapPos);
		glEnd();
	}
	
	double size = 25.0/zoom;

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, iconTexture);
	glColor3f(1,1,1);
	glBegin(GL_QUADS);

	if (selectedLoc) {
		Vector3d p = selectedLoc->pos;
		glTexCoord2f(1,  0); glVertex2d(p.x+size, p.y+2.0*size);
		glTexCoord2f(0,  0); glVertex2d(p.x-size, p.y+2.0*size);
		glTexCoord2f(0,.5f); glVertex2d(p.x-size, p.y);
		glTexCoord2f(1,.5f); glVertex2d(p.x+size, p.y);
	}
	if (!editMode) {
		if (selectedGoal) {
			Vector3d p = selectedGoal->pos;
			glTexCoord2f(1,.5f); glVertex2d(p.x+size, p.y+2.0*size);
			glTexCoord2f(0,.5f); glVertex2d(p.x-size, p.y+2.0*size);
			glTexCoord2f(0,  1); glVertex2d(p.x-size, p.y);
			glTexCoord2f(1,  1); glVertex2d(p.x+size, p.y);
		}

		list<Location*>::iterator vit = via.begin();
		for (; vit!=via.end(); vit++) {
			Vector3d p = (*vit)->pos;
			glColor3f(1,.2f,1);
			glTexCoord2f(1,  0); glVertex2d(p.x+size, p.y+2.0*size);
			glTexCoord2f(0,  0); glVertex2d(p.x-size, p.y+2.0*size);
			glTexCoord2f(0,.5f); glVertex2d(p.x-size, p.y);
			glTexCoord2f(1,.5f); glVertex2d(p.x+size, p.y);
		}
	}
	glEnd();

	glColor3f(1,1,1);

	if (!editMode) {
		unsigned numVia=(unsigned)via.size();
		char buf[16]={0};
		list<Location*>::iterator vit = via.begin();
		for (; vit!=via.end(); vit++) {
			Vector3d p = (*vit)->pos;
			snprintf(buf, 15, "%d", numVia--);
			glFont::Print(buf, (float)p.x, (float)(p.y+size*1.125), (float)(size*.65), false, true);
		}
	}

	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();

	drawStats();
}

void drawStats() {
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();

	glLoadIdentity();
	glOrtho(0, vportW, vportH, 0, -1, 1);

	glColor4f(0,0,0,.75);
	glDisable(GL_TEXTURE_2D);
	glBegin(GL_QUADS);
		glVertex2f( 260.0f, 8.0f);
		glVertex2f( 5.0f, 8.0f);
		glVertex2f( 5.0f, 26.0f);
		glVertex2f( 260.0f, 26.0f);

		if (!measurePoints.empty()) {
			glVertex2f( 230.0f, 30.0f);
			glVertex2f( 5.0f, 30.0f);
			glVertex2f( 5.0f, 48.0f);
			glVertex2f( 230.0f, 48.0f);
		}

		if (editMode && lockPointsToMap) {
			glVertex2f( vportW-5.0f, 8.0f);
			glVertex2f( vportW-111.0f, 8.0f);
			glVertex2f( vportW-111.0f, 26.0f);
			glVertex2f( vportW-5.0f, 26.0f);
		}
	glEnd();
	glEnable(GL_TEXTURE_2D);

	glColor4f(1,1,1,1);
	char tmp[128]={0};
	snprintf(tmp, 127, "GPS: %12f   %12f", mapPos.x, mapPos.y);
	glFont::Print(tmp, 10.0f, 10.0f, 14.0f);

	if (!measurePoints.empty()) {
		double sum=0;
		for (unsigned mp=1; mp<measurePoints.size(); mp++)
			sum += measurePoints[mp].distance(measurePoints[mp-1]);
		sum += measurePoints.back().distance(mapPos);

		snprintf(tmp, 127, "%s: %.4f %s", LANG["distance"].c_str(), sum, LANG["miles"].c_str());
		glFont::Print(tmp, 10.0f, 32.0f, 14.0f);
	}

	if (editMode && lockPointsToMap) glFont::Print("Points locked", vportW-105.0f, 10.0f, 14.0f);
	
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
}
