#ifndef MAP_TOOL_INCLUDED
#define MAP_TOOL_INCLUDED

#include "locations.h"
#include "connections.h"
#include "vector.h"
#include "languages.h"

#include <list>

using std::list;
using std::vector;

class Map;
class SearchInfo {
public:
	const Location* location;
	const Location* prev;
	const Connection* prCon;
	const Transportation* prTrans;

	SearchInfo(const Location* loc, const Location* pr, const Connection* con, const Transportation* tran)
		: location(loc), prev(pr), prCon(con), prTrans(tran) {}
};

extern Location *selectedLoc, *selectedGoal;
extern Connection* selectedCon;
extern int selectedWP;
extern Map* selectedMap;
extern list<Vector3d*> selectedPoints;
extern Location* startConnect;
extern list<SearchInfo> route;
extern list<Location*> via;

extern unsigned iconTexture;
extern Vector3d scrollOffset;
extern Vector3d mapPos, totalPos, totalSize;
extern double zoom;

extern int vportW, vportH;
extern bool dragLocation;
extern vector<Vector3d> measurePoints;

extern bool doForcedMarch;
extern bool fastestRoute;
extern bool lockPointsToMap;
extern bool editMode;

Vector3d win2Map(int x, int y);

void drawMap(bool showCons);
void drawStats();
void limitZoom();
void loadData(const string& file);
void planRoute(Location* from, Location* to, bool fastest);
std::string routeToString();
void centerOnRoute();

inline void clamp(float& n, float lower, float upper) {
	if (n<lower) n=lower;
	else if (n>upper) n=upper;
}

class Map {
public:
	static vector<Map> maps;
	static void save();
	static Map* load(const string& file);
	static Map* pick(const Vector3d& mp);

	string file, textureFile;
	unsigned texture;
	Vector3d position, size;
	double scale;
	bool transparent;

	Map() : transparent(false) {}
	~Map();

	void draw();
	void saveMap();
};

class TerrainType {
public:
	static vector<TerrainType> terrains;
	static void load(const string& file);
	static TerrainType* get(const string& name);

	string name;
	float mod;
	bool fastPossible;
	int id;

	TerrainType(const string& n, float m=1.0f) : name(n), mod(m), fastPossible(1), id(0) {}
};

class Transportation {
public:
	static vector<Transportation> transports;
	static void load(const string& file);

	string name;
	float speed, revSpeed, fastFac;
	unsigned excludedTerrain;
	bool allowed;
	ConType type;

	Transportation(const string& n, float s=20.0f, float f=1.0f)
		: name(n), speed(s), revSpeed(s), fastFac(f), excludedTerrain(0), allowed(0), type(LAND) {}
};

#endif
